/*****************************************************************************/
/*      EQUINOX SuperSerial Technology (SST) Driver for Linux                */
/*                                                                           */
/*      Copyright (C) 1989-2002 Equinox Systems Inc.                         */
/*****************************************************************************/
                                                                               
/* ISA FIND-A-HOLE FUNCTION (ifns.c) */
/********************************************************************************
 *										*
 *			ISA FIND-A-HOLE FUNCTION				*
 *			========================				*
 *										*
 * PURPOSE: To provide the facility to determine an ISA Controllers parameters	*
 *	    as follows:								*
 *		o Controller's physical base memory address			*
 *		o Size of memory foot print (paged (16k) or flat)		*
 *		o Controller's bus interface (8/16 bit)				*
 *										*
 *	    In order to support above functionality file contains a suite of 	*
 *	    functions including:						*
 *		o Determination of a system's unused 16k memory areas		*
 *		o Determination of addresses spaces occupied by ROM		*
 *		o Determination of a Controller's Bus Interface			*
 *										*
 * AUTHOR: BILL KRAMER								*
 * REV:    7 (6/28/94 dek)							*	
 * DATE:   04/26/94								*
 * PORTED: 04/27/94 dek
 *										*
 ********************************************************************************/


/* 
UNIX Application notes:

	This include file contains one comprehensive function, 
	namely "determ_isa_cnfg()". 

	It takes as formal input:

		port_base - port address for a located board
		isa_cnfg_parms (structure) - global structure
  \*RV8*\         ssp_type - IFNS_SSP or IFNS_SSP4
 
	It takes as global input:

		oper_input - this structure filled in from space.c variables

	It takes as global input/output:

		ist - this structure partially filled in by driver, partially
		      by this routine - status of previous call used as input


	It should be called once for each "found" isa contoller to
	determine where it should reside.  All "logic rules" are
	encapsulated in this routine.
*/

#ifdef	ISA_ENAB

/************************************************************************
 * Structure definitions & defines/literals				*
 ************************************************************************/

#include <linux/version.h>
/*Literals used when searching for ROM											*/
#define ROM_BNDRY_SZ	0x800					/*Size (in bytes) to next possible ROM boundary		*/
#define ROM_AREA_BASE	0xC0000					/*Base address of area that can be occupied by ROM	*/
#define ROM_AREA_LMT	0xF4000					/*Address of top of ROM Area + 2k			*/
#define MAX_ROM_LOCS	((unsigned int)((ROM_AREA_LMT-ROM_AREA_BASE)/ROM_BNDRY_SZ)) /*# of possible ROM starting locs	*/

#define	ISA_ADDR_LOW	((int) 0x00)				/* Write (16-bits), addr bits 21 -14 */
#define ISA_PAGE_SEL	((int) 0x02)				/* write (16-bits), page select if page mode */

/*ICP displacements													*/
#define ICP_GLOBAL_DISPLC   ((unsigned int)0x2000)		/*Displacement from base of ICP regs to Global regs	*/

#define EQNX_VERSION_CODE LINUX_VERSION_CODE

#if (EQNX_VERSION_CODE < 131328)  /* kernel 2.0.x and below */
#define ioremap vremap
#define iounmap vfree
#endif

/************************
 *  Public Data Areas	*
 ************************/

	unsigned char			ux_ifns_rev = 0x05 ;		/*File's revision level				*/

	unsigned char			dynamic_cnfg_init = 0 ;		/*Dynamic configured ISA Controller flag:	*/
									/*	o 0x00 = Have not dynamically configured*/
									/*		 a controller			*/
									/*	o 0xFF = Have dynamically configured a	*/
									/*		 controller and structure	*/
									/*		  "dynamic_config" is init'd	*/

	struct cntrl_cfig    		dynamic_cnfg_struct ;		/*Parameters if ISA Controller Dynamically	*/
									/*configured.					*/
									/*NOTE: o Structure only valid if 		*/
									/*	  "dynamic_cnfg_cmplt" = 0xFF		*/
									/*	o Element "base_port" in structure is	*/
									/*	  ignored and therefore is assumed 0	*/

/*Declare structure used to pass Installation information as specified by operator (in "space.c")			*/

struct operator_parms  oper_input = {
								/* USER INPUTS */
	(unsigned char)		( 0 ) ,				/*System's Bus Type					*/
	(unsigned char)		( 0 ) ,				/*Default baud rate table				*/
	(unsigned char)		( 1 ) ,				/*Quiet Mode flag					*/
	(unsigned char)		( 0 ) ,				/*Number of Host Controllers				*/

	(unsigned short int)	( MAX_ISA_ARRAY_SIZE ) ,	 /*Number of valid elements in ISA Hole Table		*/
	  			{				/*Table of 16k memory hole Physical Addresses to check	*/
								/*NOTE: Table arranged from most probable memory hole	*/
								/*	location to least probable.			*/

								/*ARRAY INDEX (decimal)					*/
					0xD0000,		/*     00						*/
					0xB4000,		/*     01						*/

					0xB0000,		/*     02						*/
					0xD4000,		/*     03						*/
					0xD8000,		/*     04						*/
					0xDC000,		/*     05						*/

					0xE0000,		/*     06						*/
					0xE4000,		/*     07						*/
					0xE8000,		/*     08						*/
					0xEC000,		/*     09						*/

					0xC0000,		/*     10						*/
					0xC4000,		/*     11						*/
					0xC8000,		/*     12						*/
								        
					0xB8000,		/*     13						*/
					0xBC000,		/*     14						*/

					0xCC000,		/*     15						*/

					0xA0000,		/*     16						*/
					0xA4000,		/*     17						*/
					0xA8000,		/*     18						*/
					0xAC000,		/*     19						*/
#ifndef LINUX

/*RV7*/					0x80000,		/*     20						*/
/*RV7*/					0x84000,		/*     21						*/
/*RV7*/					0x88000,		/*     22						*/
/*RV7*/					0x8C000,		/*     23						*/

/*RV7*/					0x90000,		/*     24						*/
/*RV7*/					0x94000,		/*     25 						*/
/*RV7*/					0x98000,		/*     26						*/
/*RV7*/					0x9C000			/*     27 						*/
#endif
				      } ,			/*End of Memory Hole Table				*/

#ifndef LINUX
	(unsigned char)		( 1 ) ,				/*Bit 31 Memory Check flag				*/
#else
	(unsigned char)		( 0 ) ,				/*Bit 31 Memory Check flag				*/
#endif
	(unsigned int)		( MAX_BIT_31_ARRAY_SIZE-1 ) ,	/*Number valid elements in bit 31 table			*/
				{				/*Table of 16k mem hole Physc Addrs w/bit 31 set	*/
				      /*NOTE: 0x80000000 can not be used since appears to shadow low 1M of memory	*/

								/*ARRAY INDEX (decimal)					*/
					0x80100000,		/*     00						*/
					0x80200000,		/*     01						*/
					0x80300000,		/*     02						*/
					0x80400000,		/*     03						*/
					0x80500000,		/*     04						*/
					0x80600000,		/*     05						*/
					0x80700000,		/*     06						*/
					0x80800000,		/*     07						*/
					0x80900000,		/*     08						*/
					0x80A00000,		/*     09						*/
					0x80B00000,		/*     10						*/
					0x80C00000,		/*     11						*/
					0x80D00000,		/*     12						*/
					0x80E00000,		/*     13						*/
					0x80F00000,		/*     14						*/
				      } ,			/*End of bit 31 memory hole array			*/

								/* USER INPUTS */
/*RV7*/	(unsigned char)		( 1 ) ,				/*VGA 8-bit memory Override 				*/
	(unsigned char)		( 0 ) ,				/*Default Controller parameter spec'd			*/
				{ 0 } ,				/*Default Controller parameters				*/
        (unsigned char)		( 0 ) ,				/*Number of entries in explicit Configuration table	*/
             			{{ 0x200, 0x80104000l, 0, 0xFF }}	/*Table entry						*/
	} ;  /* oper_input */

/********************
 * Static Data Area *
 ********************/

/* Flag indicating if ROM Array below has been          */
/*initialized/built:					*/
/*	o 0x00 = Array not initialized			*/
/*	o 0xFF = Array initialized			*/

static  unsigned char		rom_array_built = 0;		

/*Byte mapped array of memory areas occupied by ROM	*/
/*NOTE: o Initialized by "FIND 16k MEMORY HOLE" function*/
/*	o Element 0 = ROM_AREA_BASE			*/
/*	o Each element/index = 0x800 (2k)		*/
/*	o Last element = ROM_AREA_LMT			*/
/*	o Element value 0x00 = No ROM			*/
/*	o Element value 0xFF = ROM present		*/
static	unsigned char		rom_array[MAX_ROM_LOCS] ;	


/************************************************************************************/
static unsigned char determ_isa_cnfg( unsigned short int base_port, 
         struct isa_cnfg_parms *ret_parms, unsigned char ssp_type); 
static unsigned char    determine_bus_interface  (unsigned short int,
unsigned long int, unsigned char *, unsigned char) ;

/*Find 16k Memory Hole Function		*/
static unsigned char	find_memory_hole (unsigned char, unsigned long int *);
static unsigned char occupied_check (unsigned long int physc_base_addr );
static unsigned char passive_occupied_chk (unsigned long int physc_addr,
	unsigned char *log_addr);
static unsigned char	rom_check (void) ;

/************************************************************************************/
static unsigned char determ_isa_cnfg( unsigned short int base_port, 
         struct isa_cnfg_parms *ret_parms, unsigned char ssp_type)
{
        volatile struct cntrl_cfig      *found_config_ptr = NULL ;
        unsigned long int               physc_addr ;
        unsigned long int               page_base ;
        unsigned int                    mem_index, i ;
        unsigned char                   vga_rom_found;
        unsigned int                    oride_index ;
        unsigned char                   uchar_ret_val;

        ret_parms->cntrlr_parms.base_port = base_port;
        ret_parms->method = 3;
        if ( (oper_input.sys_bus_type != 0x01) &&
             (oper_input.sys_bus_type != 0x02)){
                printk("bustype is not isa or eisa\n");
                ret_parms->status = 0x02 ;
                return (ret_parms->status) ;
        }
        if ( (ssp_type != IFNS_SSP) &&
                (ssp_type != IFNS_SSP4) ){
                printk("ssp type is not valid\n");
                ret_parms->status = 0x02 ;
                return (ret_parms->status) ;
        }
	
/*Set up & determine Controller's settings (parms.) as follows:								*/
/*	1. Check for Controller's I/O port in explicit table (see Operator Parameter structure)				*/
/*	2. Check defaut entry in Operator Parameter structure								*/
/*	3. Check if an explicit entry exists that is between 640k & 1M that is paged					*/
/*	4. Check if an explicit entry exists that is paged								*/
/*	5. Check if a configuration has been prevously dynamically determined						*/
/*NOTE: If none of the above are found then attempt to dynamically identify parameters for callers controller		*/

	for ( i = 0; i < oper_input.isa_explct_ents; ++i )	/*Search for caller's Controller in Explicit tbl	*/
	{
		if ( oper_input.isa_explct_tbl[i].base_port == base_port ) /*Did operator explicitly config controller ?*/
		{							   /*If yes, pt to entry			*/
			found_config_ptr = &(oper_input.isa_explct_tbl[i]) ;   	/*Set up ptr to Controller's parms	*/

		/*Common exit point - Controller's configuration parms identified					*/
		/*NOTE: found_config_ptr points to parms								*/
EXIT:
			ret_parms->cntrlr_parms.cntrlr_base_addr  = found_config_ptr->cntrlr_base_addr ;
			ret_parms->cntrlr_parms.cntrlr_mode       = found_config_ptr->cntrlr_mode ;
			if ( found_config_ptr->cntrlr_mode == 0x40 )	   /*Is Controller configured in Flat mode ?	*/
				ret_parms->cntrlr_parms.cntrlr_bus_inface = 0x20 ;	/*Yes, bus interface = 16 bits	*/
			else						   		/*No, Controller in page mode	*/
			{
				if ( found_config_ptr->cntrlr_bus_inface != 0xFF )	/*Determ interface i.e. dynamic?*/
				{							/*If no, return interface	*/
					ret_parms->cntrlr_parms.cntrlr_bus_inface = found_config_ptr->cntrlr_bus_inface ;
				}
				else							/*If yes, go dynamically determ	*/
				{							/*bus interface			*/
					uchar_ret_val = determine_bus_interface (base_port,	/*Determ Ctrlr's intrfce*/
								 	     found_config_ptr->cntrlr_base_addr,
								 	     &(ret_parms->cntrlr_parms.cntrlr_bus_inface),
/*RV8*/                      ssp_type);
					if ( uchar_ret_val != 0x00 )			/*Successful determination?	*/
					{						/*If no, ret with error 	*/
					ret_parms->status = uchar_ret_val ;
					return (ret_parms->status) ;
					}
				}
			}
			ret_parms->status = 0x00 ;
			return (ret_parms->status) ;				/*Return to caller with found status	*/
		}							/*End of found cntrlr's entry in explicit tbl	*/
	}							/*End of explicit table search (for loop)		*/



	if ( oper_input.isa_def_specd == 0xFF )			/*Did operator specify default controller parms ?	*/
	{							/*If yes, pt to entry					*/
		found_config_ptr = &(oper_input.isa_def_config) ;	/*Set up ptr to Controller's parms		*/
		goto EXIT ;						/*Go return to caller with parms		*/
	}
        /*Clear variable indicating no paged entry found*/
        found_config_ptr = (struct cntrl_cfig *)NULL;
	for ( i = 0; i < oper_input.isa_explct_ents; ++i )	/*Search for implicit parms by checking Explicit table	*/
	{							/*for paged entry that is between 640 & 1M		*/
		if ( oper_input.isa_explct_tbl[i].cntrlr_mode == 0x00 ) /*Does explicit entry describe a 16k area ?	*/
		{						/*If yes, use as default for non-specified controllers	*/
			if ( (oper_input.isa_explct_tbl[i].cntrlr_base_addr >= 0x0A0000l) &&
			     (oper_input.isa_explct_tbl[i].cntrlr_base_addr <  0x100000l)    )
			{						/*If found paged entry between 640k & 1M	*/
				found_config_ptr = &(oper_input.isa_explct_tbl[i]) ;   	/*Set up ptr Controller's config*/
				goto EXIT ;						/*Go return with parms		*/
			}
			if ( found_config_ptr == (struct cntrl_cfig *)0 )		/*Have already identified entry?*/
				found_config_ptr = &(oper_input.isa_explct_tbl[i]) ;	/*If no, save in case can't find*/
											/*entry betwn 640k & 1M		*/
		}							/*End of found cntrlr's entry in explicit tbl	*/
	}							/*End of implicit search  (for loop)			*/
	if ( found_config_ptr != (struct cntrl_cfig *)0 )	/*Did find at least find an explicit paged entry ?	*/
		goto EXIT ;					/*If yes, use it					*/
	ret_parms->method = 2;
	if ( dynamic_cnfg_init == 0xFF )			/*Has a set of dynamic parms been previously extablish ?*/
	{							/*If yes, set caller controller's parms the same	*/
		found_config_ptr = &dynamic_cnfg_struct ;		/*Set up ptr to Controller's parms		*/
		goto EXIT ;						/*Go return to caller with parms		*/
	}

/*As result of searching Operator/Installation explicit Controller parameters, default parameters, and previously 	*/
/*configured Controller dynamic parameters, have not found a suitable set of parms for caller's controller, therefore	*/
/*set up & DYNAMICALLY determine controller parms by searching address tables for an available 16k memory hole		*/

        /*Identify avail 16k mem areas*/
        uchar_ret_val = find_memory_hole(oper_input.sys_bus_type, &page_base); 
        if ( uchar_ret_val != 0 ){
                /*If no, return to caller indicating unable to config.  */
                printk("could'nt find any memory hole\n");
                ret_parms->status = uchar_ret_val ;
                return (ret_parms->status); /*Return error status to caller*/
        }
        for (mem_index = 0 ; ist.sys_isa_hole_status[mem_index] != 0;
             ++mem_index ){
		unsigned char cntlr_bus_inface;

                if ( ist.sys_isa_hole_status[mem_index] == 0xF0 ){
                        physc_addr = ist.sys_isa_holes[mem_index];
                        /*Get area's bus interface size         */
                        uchar_ret_val = determine_bus_interface (base_port,
                                physc_addr, 
                                &cntlr_bus_inface, 
                                ssp_type);
                        /*Bus interface determation successful?         */
                        if ( uchar_ret_val != 0x00)
                                /*If no, mark mem area unavail  */
                                /*and try next memory area      */
                                ist.sys_isa_hole_status[mem_index] = 
					uchar_ret_val;
			else{
                                if (cntlr_bus_inface == 0x20)
                                        ist.sys_isa_hole_status[mem_index] = 	
						0xF3 ;
				else
                      			/*16k memory area functional 
                      			w/8 bit bus intface*/
                      			/*Mem avail w/8 bit intface     */
                                        ist.sys_isa_hole_status[mem_index] = 	
						0xF1;
			}
		}
	}
        /*Loop searching for 16k page of unoccupied memory that */
        /*can be used with a 16 bit bus interface               */
        for (mem_index = 0 ; ist.sys_isa_hole_status[mem_index] != 0;
             ++mem_index ){                                                     
                /*Is 16k mem area (addr) available ?            */
                if ( ist.sys_isa_hole_status[mem_index] == 0xF3 ){

                        ist.sys_isa_hole_status[mem_index] = 0xFF ;
                	dynamic_cnfg_struct.cntrlr_bus_inface = 0x20;

                        /*Mem accessable w/16 bit inerface?*/
                        /*If yes, set up dynamic config */
                        /*parms for future use          */
       
                        /*Null base port                */
                        dynamic_cnfg_struct.base_port = 0x00 ;
                        /*Dynamic 16k mem area addr     */
                        dynamic_cnfg_struct.cntrlr_base_addr = 
                        	ist.sys_isa_holes[mem_index];
                        /*Dynamic 16k mem mode = pg     */
                        dynamic_cnfg_struct.cntrlr_mode = 0x00 ;
                        /*Bus interface est. via fn     */
                        /*Indicate Dynam Cnfg init'd*/
                        dynamic_cnfg_init = 0xFF ;                          
                        /*Indicate mem area alloc'd     */
                        /*Ptr to Cntrlr's parms */
                        found_config_ptr = &dynamic_cnfg_struct ;           
                        /*Go return with parms  */
                        goto EXIT ;                                         
                 }       /*End of mem accessable w/16 bit        */
        }/*End of "for" loop srch'g for avail mem area w/16 bit intrfce  */

        /*Did not successfully find an avail 16k mem area with a 16 bit bus interface, 
                check if should attempt VGA ROM 8-bit   */
        /*region (memory area) override                                                                                         */
        /*NOTE: - Any available memory holes at this point should have an 8 bit interface                                       */
        /*Should attempt to override 8 bit VGA ROM Memory ?             */

        if ( oper_input.vga_hole_override != 0xFF){
                /*If yes */
                /*Loop searching for VGA ROM at 0xC0000                 */
                for (mem_index = 0, vga_rom_found = 0 , 
                        oride_index = 0xFFFFFFFF ;
                        ( (physc_addr = ist.sys_isa_holes[mem_index]) != 0x00 ) ;
                        ++mem_index ){
                        if ( ((physc_addr & 0xFFFFF) >= 0xC0000) &&     /*Is table entry in VGA ROM 128k region ?       */
                                ((physc_addr & 0xFFFFF) <  0xE0000) ){
                                /*If yes, chk if hole = 8 bit RAM or VGA ROM    */
                                if ( (physc_addr & 0xFFFFF) == 0xC0000){
                                        /*Is mem area = VGA ROM addr space ?    */
                                        /*If yes, check if VGA ROM present      */
                                        /*If VGA ROM not present,       */
                                        /*Don't attempt override        */
                                        if ( ist.sys_isa_hole_status[mem_index] != 0x01 ) 
                                                break ;                                   
                                        /*Indicate system has VGA ROM           */
                                        vga_rom_found = 0xFF ;                  
                                }
                                else{
                                        /*Memory area not VGA ROM, validate     */
                                        /* some other ROM or 8 bit hole         */
                                        if (ist.sys_isa_hole_status[mem_index] != 0x01){
                                                /*Is memory area = ROM?         */
                                                /*If no, chk if 8 bit hole      */
                                                /*Is mem area = 8 bit   */ 
                                                if ( ist.sys_isa_hole_status[mem_index] == 0xF1){
                                                        /*If yes                */
                                                        /*Is Override addr estab?*/
                                                        if ( oride_index == 0xFFFFFFFF )
                                                                oride_index = mem_index ;       
                                                        /*If no                 */
                                                }
                                                else{
                                                        /*Mem area not ROM & not*/
                                                        /*an 8 bit hole, don't  */
                                                        /*attempt override      */
                                                        oride_index = 0xFFFFFFFF;               
                                                        break ;
                                                }
                                        }
                                }
                        }/*End of IF used to chk holes in VGA ROM region */
                }/*End of VGA ROM 8-bit hole area override FOR loop     */

                /*Did successfully find a 16k memory hole in VGA ROM    */
                /* region that can be used in 16 bit mode thus          */
                /*  overriding VGA ROM's 8 bit interface ??             */
                if ( (physc_addr == 0) && (vga_rom_found == 0xFF) &&            
                        (oride_index != 0xFFFFFFFF)){
                        /*If yes, use 16k memory area indicating override       */
                        /*Indicate mem area alloc'd     */
                        ist.sys_isa_hole_status[oride_index] = 0xFD ;     
                        /*Set base port = null          */
                        dynamic_cnfg_struct.base_port         = 0x00 ;                  
                        /*16k mem area addr     */
                        dynamic_cnfg_struct.cntrlr_base_addr  = 
                                ist.sys_isa_holes[oride_index]; 
                        /*16k mem mode = pg     */
                        dynamic_cnfg_struct.cntrlr_mode = 0x00;                         
                        /*Bus interfce = 16 bits*/
                        dynamic_cnfg_struct.cntrlr_bus_inface = 0x20 ;                          
                        /*Dynam Cnfg initialized*/
                        dynamic_cnfg_init = 0xFF ;                                              
                        /*Ptr to Cntrlr's parms */
                        found_config_ptr = &dynamic_cnfg_struct ;                               
                        /*Go return with parms          */
                        goto EXIT ;                                                             
                }
        }       /*End of VGA ROM 8-bit memory area override check (IF) */
/*Did not successfully find an avail 16k mem area with a 16 bit bus interface, check if an 16k mem area exists with an	*/
/*8 bit bus interface													*/
	for (mem_index = 0 ;					/*Loop searching for 16k page of unoccupied memory that	*/
	     ( ist.sys_isa_hole_status[mem_index] != 0xF1 ) &&	/*can be used with an 8 bit bus interface		*/
	     ( ist.sys_isa_hole_status[mem_index] != 0x00 ) ;
	     ++mem_index ) ;
	if ( ist.sys_isa_hole_status[mem_index] == 0xF1 )	/*Did find a memory area w/8 bit bus interface ?	*/
	{							/*If yes, use area					*/
		ist.sys_isa_hole_status[mem_index] = 0xFE ; 	    			/*Indicate mem area alloc'd	*/
		dynamic_cnfg_struct.base_port         = 0x00 ;				/*Set base port = null		*/
		dynamic_cnfg_struct.cntrlr_base_addr  = ist.sys_isa_holes[mem_index]; 	/*Dynamic 16k mem area addr	*/
		dynamic_cnfg_struct.cntrlr_mode       = 0x00 ;	    			/*Dynamic 16k mem mode = pg	*/
		dynamic_cnfg_struct.cntrlr_bus_inface = 0x00 ;				/*Bus interface = 8 bits	*/
		dynamic_cnfg_init = 0xFF ;			    			/*Indicate Dynam Cnfg init'd	*/
		found_config_ptr = &dynamic_cnfg_struct ;	    			/*Ptr to Cntrlr's parms		*/
		goto EXIT ;					    			/*Go return with parms		*/
	}

/*Did not successfully find an avail 16k mem area with an 8 bit bus interface, return to caller with not found status	*/
	ret_parms->status = 0xFF ;
	return (ret_parms->status) ;
}

/****************************************************************************************
 *                                                                              	*
 *				DETERMINE BUS INTERFACE SIZE				*
 *                                                                              	*
 * PURPOSE: For a given 16k memory area, determine its bus interface size (8/16 bits)	*
 *											*
 * NOTES: 										*
 *											*
 * CALL:  unsigned char    determine_bus_interface  (unsigned short int	 base_port,	*
 *						     unsigned long int	 physc_addr,	*
 *						     unsigned char	 *size      )	*
 * \*RV8*\         unsigned char   ssp_type)      *
*											*
 *			where: base_port  = Controller's base I/O port			*
 *			       physc_addr = Physical addr of 16 area			*
 *			       *size      = Pointer for bus interface size return parm:	*
 *						o 0x00 =  8 bit bus interface		*
 *						o 0x20 = 16 bit bus interface		*
 *											*
 *											*
 * RETURN:  - Status (unsigned char):							*
 *			o 0x00 = Function successfully determined memory areas bus	*
 *				 interface size ("size" return parameter valid)		*
 *			o 0x03 = Function unsuccessful due to memory area appearing to	*
 *				 be cached ("size" return parameter invalid)		*
 *			o 0x04 = Function unsuccessful due to a conflict, e.g. can not 	*
 *				 successfully read ICP's "channel counter" or bad base	*
 *				 I/O port ("size" return parm invalid).			*
 *			o 0x80 = Function unsuccessful due to memory mapping failure	*
 *				 ("size" return parameter invalid)			*
 *											*/
static unsigned char    determine_bus_interface  (unsigned short int   base_port,
                unsigned long int physc_addr, unsigned char     *size , 
                unsigned char ssp_type)

{
        /*Variables used to determine bus interface size */
        /*Logical base addr of 16k memory area (ICP ptr)        */
        struct icp_gbl_struct        *log_icp_ptr ;          
        /*Mod 128k physical base addr that caller's memory area */
        unsigned long int               share_area_addr ;       
        /*resides in 16 bit read of ICP ATTN & Chan Ctr regs            */
        unsigned short int              attn_chan ;
        /*16 bit rd of ICP ATTN/Chan Ctr regs used for chan chnge*/
        unsigned short int              attn_chan_chng ;        

        /*General purpose variables */
        unsigned char                   uchar_ret_val ; 
        unsigned char                   return_status ;
        unsigned int i;
        void *  caddr;
        int ii; 
        /*Logical base addr of 16k memory area (SSP4 ptr)*/
        volatile struct ssp2_global_s  *log_ssp2_ptr ;         
        unsigned short int              cur_chnl;
        unsigned short int w;
#if (EQNX_VERSION_CODE < 66304)
        int rc;
#endif
        log_ssp2_ptr = NULL; /* get rid of compiler warning */
        log_icp_ptr = NULL;  /* get rid of compiler warning */

        if ( ssp_type == IFNS_SSP )
                physc_addr += ICP_GLOBAL_DISPLC;
#if (EQNX_VERSION_CODE > 66304)
		
#if (EQNX_VERSION_CODE < 131328)
	if (physc_addr > 0x100000){
#endif
		if ((caddr = ioremap(physc_addr, 0x1000)) == (void *) NULL){
                	printk("EQUINOX: determine_bus_interface: Failed to map physical address space = %x\n",
                        (unsigned int) physc_addr); 
                	return((unsigned char)0x80);
        	}
#if (EQNX_VERSION_CODE < 131328)
	}else
		caddr = (void *) physc_addr;
#endif		
#else
        if((caddr = vmalloc(0x1000)) == (char *) NULL){
                printk("EQUINOX: Failed to allocate virtual address space during ISA config\n");
                return((unsigned char)0x80);
        }
        if ((rc = remap_page_range((TASK_SIZE + ((unsigned long) caddr)),
                physc_addr, 0x1000, PAGE_IOMEM))) {
                printk("EQUINOX: Failed to map physical address space = %x, errno = %d\n",
                        (unsigned int) physc_addr, rc); 
                return((unsigned char)0x80);
        }
#endif
        /*Did successfully map address ?*/
        /*If no, exit with error status*/
        if ( !caddr )
                return ((unsigned char)0x80) ;

        if ( ssp_type == IFNS_SSP ) {
                log_icp_ptr = (struct icp_gbl_struct *) caddr;
        /*Disable ICP window in case enabled                    */
        outw( 0x0000, base_port + ISA_PAGE_SEL ) ;

        /*Port 1 = pg mode + 8 bit bus intface + addr bits 23 22*/
        /*Port 0 = addr bits 21 - 14                          */
        /*Init. Controller in page mode at caller's physc addr  */
        outw( ( ((physc_addr & 0xC00000) >> (22-8)) + 0x0000) |
                 ((physc_addr & 0x3FC000) >> 14),
                base_port + ISA_ADDR_LOW);

        /*Enable ICP window*/
        outw( 0x100, base_port + ISA_PAGE_SEL ) ;
        /*Set ICP = Dram steer + 8 bit bus                      */
        log_icp_ptr->gicp_bus_ctrl = 0x08 ;

        /* TEMP for fixed hardware condition */
        for ( ii = 512; ii != 0; --ii )
                if ( log_icp_ptr->gicp_bus_ctrl == 0x08 )
                  break;
        if ( ii == 0 ) {
          return_status = 4;
          goto DETERMINE_BUS_EXIT;
        }

    /*determine if area functional with 8 bit bus interface*/
    /*NOTE: Controller & ICP set for 8 bit interface*/

        /*Disable addressing, so can't find in 128k srch        */
        outw( 0x0000,base_port + ISA_PAGE_SEL) ;
        /*Base of 128k region caller's address resides in*/
        /*Set up to determine if 128k region that caller's      */
        /*mem area resides in is occupied with some other memory*/
        share_area_addr = physc_addr & 0xFFFE0000 ;
        for (i = 0 ; (i < 8) &&
             ( occupied_check(share_area_addr+(i*0x4000) ) == 0xF0) ;
             ++i        ) ;
        /*Enable ICP addressing again                           */
        outw( 0x100, base_port + ISA_PAGE_SEL) ;
/*-------------------------------------------*/
 }
 else  /* IFNS_SSP4 */
 {
   log_ssp2_ptr = (struct ssp2_global_s *) ((unsigned char *)caddr + 0x400);

   /*Disable ICP window, set paged and memory-mapped*/
   outb( 0x00, base_port + SSP4_PAGE_SEL ) ;

   /*addr bits 14 - 15, disable ints */
   /*Init. Controller in page mode at caller's physc a ddr  */
   outb((((physc_addr & 0x0000C000) >> 8) & 0xFF),
          (base_port + SSP4_ADDR_LOW));


   /*Port 0 = addr bits 21 - 14 */
   /*Init. Controller in page mode at caller's physc addr  */
   outb(  (((physc_addr & 0xFFFF0000) >> 16) & 0xFF),
          (base_port + SSP4_ADDR_HIGH));

   /*Enable ICP window, base ICP*/
   outb( 0x01, (base_port + SSP4_PAGE_SEL));
   /*Set ICP = 8 bit bus*/
   log_ssp2_ptr->bus_cntrl = 0;

 /*determine if area functional with 8 bit bus interface*/
 /*NOTE: Controller & ICP set for 8 bit interface
    */
   /*Disable addressing, so can't find in 128k srch */
   outb(0x00, (base_port + SSP4_PAGE_SEL)) ;
   /*Base of 128k region caller's address resides in */
   /*Set up to determine if 128k region that caller's*/
   /*mem area resides in is occupied with some other memory*/
   share_area_addr = physc_addr & 0xFFFE0000 ;
   for (i = 0 ; (i < 8) &&
        ( occupied_check(share_area_addr+(i*0x4000) ) == 0xF0) ;
        ++i        ) ;
   /*Enable ICP addressing again*/
   outb( 0x01, (base_port + SSP4_PAGE_SEL));
 }

/*-------------------------------------------*/
        /*Is any other mem present in 128k addr space ?         */
        /*If no, caller's mem area can use 16 bit bus interface */
        /*Set up bus interface return parm = 16 bit     */
        /*Set up function's return status               */
        /*Return to caller*/
        if ( i >= 8 ){
                *size = 0x20 ;
                return_status = 0x00 ;
                goto DETERMINE_BUS_EXIT ;
        }


        /* is it system ROM? */
        if ( share_area_addr == 0xe0000 ) {
                /* is 0xe0000 to 0xeffff free? */
                if ( i >= 4 ){
                /* yes - set to 16 bits */
                /*Set up bus interface return parm = 16 bit     */
                /*Set up function's return status               */
                /*Return to caller                              */
                *size = 0x20 ;
                return_status = 0x00 ;
                goto DETERMINE_BUS_EXIT ;
                }       
        }

    /*16k memory area resides in a 128k memory region that is not empty, determine if 128k region is occupied by memory */
    /*using an 8 bit bus interface.  Perform this check by issuing a 16 bit read operation with Controller & ICP in     */
    /*8 bit mode.                                                                                                       */

 if ( ssp_type == IFNS_SSP ){

        /*Issue 16 bit mem read of ICP regs             */
        attn_chan = *((unsigned short int *)(&log_icp_ptr->gicp_attn));
        /*Does 16 bit read appear to have functioned ?  */
        if ( ((attn_chan & 0xC0F0) == 0) &&
            ((*((unsigned short int *)(&log_icp_ptr->gicp_attn)))&0xC0FF) == (attn_chan & 0xC0FF))
        {
            /*If yes, validate can "see" ICP's chan ctr ticking  */
            /*NOTE: Chan ctr is hi byte of 16 bit read           */
                for (i = ((unsigned long int)((260000*2)/(10*8))) ;
                     i != 0 ; --i ) {
                        /*ICP's chan ctr in hi byte*/
                        attn_chan_chng = *((unsigned short int *)(&log_icp_ptr->gicp_attn));
                        /*Did val chng frm 1st rd? */
                        if ( (attn_chan_chng & 0x3F00) != (attn_chan & 0x3F00) )
                                /*If yes, 8 bit mode ok    */
                                break ;
                }
                /*Did successfully read 16 bits in 8 bit mode?       */
                if ( i != 0){
                    /*If yes, implies other memory in same 128k region is*/
                    /*functioning with an 8 bit bus interface            */
                        /*Set up bus interface return parm = 8 bit      */
                        /*Set up function's return status               */
                        /*Return to caller                              */
                        *size = 0x00 ;
                        return_status = 0x00 ;
                        goto DETERMINE_BUS_EXIT ;
                }
        }

  }
        else  /* IFNS_SSP4 */
  {
        /*Issue 16 bit mem read of ICP regs */
       
          w = * ((unsigned short int *) (&log_ssp2_ptr->filler3));

          if ( w == 0x0100 ){
          /* is gssp4_rev == 1 when read in 8-bit mode? */
          /* Issue 16 bit mem read */
             cur_chnl = * ((unsigned short int *) (&log_ssp2_ptr->unused_3)); 
            
             if ( !(cur_chnl & 0xc000) ) {
                 /*If yes, validate can "see" ICP's chan ctr ticking  */
                /*NOTE: Chan ctr is hi byte of 16 bit read */
                for (i = ((unsigned long int)((260000*2)/(10*8))) ;
                   i != 0 ;
                   --i ) {
                    /*ICP's chan ctr in hi byte */
                   w = * ((unsigned short int *) (&log_ssp2_ptr->unused_3)); 
                   /*Did val chng frm 1st rd? */
                   if ( (w & 0x3F00) != (cur_chnl & 0x3F00) )
                            /*If yes, 8 bit mode of */
                            break;
                 }
                 /*Did successfully read 16 bits in 8 bit mode ?*/
                 /*If yes, implies other memory in same 128k region is */
                        /*functioning with an 8 bit bus interface */
                 if ( i != 0 ){
                      *size = 0x00 ; 
                        /*Set up bus interface return parm = 8 bit */
                    /*Set up function's return status */
                    /*Return to caller */
                    return_status = 0x00 ;
                    goto DETERMINE_BUS_EXIT ;

                 }
             }
     }
   }
 


    /*16 byte read of ICP's Channel Counter & Attention registers failed indicating that caller's 16k memory area       */
    /*resides in a 128k region with memory using a 16 bit bus interface.  Return to caller indicating 16k memory area   */
    /*requires a 16 bit bus interface                                                                                   */
        *size = 0x20 ;                                          /*Set up bus interface return parm = 16 bit             */
        return_status = 0x00 ;                                  /*Set up function's return status                       */



    /*Function's exit point, disable Controller & free up memory area                                                   */
    /*NOTE: return_status = function's return status parameter                                                          */

DETERMINE_BUS_EXIT:
 if ( ssp_type == IFNS_SSP ){

        if ( return_status != 4 )
          log_icp_ptr->gicp_bus_ctrl = 0;

        /*Disable Controller's addressing                       */
        outw( 0x0000, base_port + ISA_PAGE_SEL ) ;
        outw( 0x0000, base_port + ISA_ADDR_LOW) ;
#if ((EQNX_VERSION_CODE > 66304) && (EQNX_VERSION_CODE < 131328))
	if (physc_addr > 0x100000)
#endif
	  iounmap(caddr);
 }
 else  /* IFNS_SSP4 */
 {
   if ( return_status != 4 )
     log_ssp2_ptr->bus_cntrl = 0;

   /*Disable Controller's addressing*/
   outb(0x00, (base_port + SSP4_PAGE_SEL) ) ;
   /* clear physical addr */
   outb(0, (base_port + SSP4_ADDR_LOW) );
   /* clear physical addr */
   outb(0,(base_port + SSP4_ADDR_HIGH) );
#if ((EQNX_VERSION_CODE > 66304) && (EQNX_VERSION_CODE < 131328))
	if (physc_addr > 0x100000)
#endif
	  iounmap(caddr);
 }


        /* no return code */
        uchar_ret_val = 0;
        /*Did successfully release memory area ?                */
        /*If no, indicate failure reason                        */
        if ( uchar_ret_val != 0 )
                return_status = 0x80 ;
        return (return_status) ;

}  /* determine_bus_interface */
/****************************************************************************************
 *                                                                              	*
 *				FIND 16k MEMORY HOLE					*
 *                                                                              	*
 * PURPOSE: To indentify 16k unused areas of memory (memory hole) on an ISA or EISA	*
 *	    system and build arrays ("sys_isa_holes" and "sys_isa_hole_status" in 	*
 *	    structure "ist_struct") indicating results of identification.  		*
 *											*
 * NOTES: o Function unconditionally DISABLES/ENABLES INTERRUPTS !!!!!!			*
 *											*
 * CALL:  unsigned char    find_memory_hole  (unsigned char 	   bus_type,		*
 *					      unsigned long int	   *avail_addr)		*
 *											*
 *			where: bus_type   = System bus type:				*
 *						o 1 = ISA (perform bit 31 testing)	*
 *						o 2 = EISA (don't perform bit 31 testng)*
 *						o ? = Any other value, function returns	*
 *						      immediately with value of 0x03	*
 *			       avail_addr = Ptr to location that function uses to return*
 *					    32 bit physical addr of unoccupied 16k area	*
 *					    (i.e. physical addr of a found memory hole)	*
 *											*
 *											*
 * RETURN:  - Status (unsigned char):							*
 *			o 0x00 = Function successfully found memory hole with		*
 *				 NOTE: "avail_addr" = 32 bit physical addr of available	*
 *					memory area					*
 *			o 0x02 = Unrecognized sytem bus parameter ("bus_type" invalid)	*
 *				 ("avail_addr" = 0).					*
 *			o 0x80 = Function unsuccessful due to not being able to map	*
 *				 addresses ("avail_addr" = 0).				*
 *			o 0xFF = No available memory holes ("avail_addr" = 0).		*
 *	    - Arrays "sys_isa_holes" and "sys_isa_hole_status" in structure "ist_struct"*
 *	      (see ist.h) are built and returned in "ist" area				*
 *											*
 ****************************************************************************************/

static unsigned char	find_memory_hole (unsigned char bus_type,
		  	  unsigned long int   *avail_addr )

{

/*Adrress used to test system's bit 31 addressing */
#define	B31_PHYSC_ADDR	0x80000000				
/*Physc addr 0, used to test system's bit 31 addressing	*/
#define	PHYSC_ADDR_0	0x0					


/*Variables used to probe for memory holes */
#if (EQNX_VERSION_CODE < 66304)
	int rc = 0;
#endif
	unsigned int pa_index ; /*Probe array index*/

/*Variables used to check if system supports bit 31 addressing*/

/*Logical addr used to access physc. addr w/bit 31 set	*/
	unsigned long int	*log_bit_31_addr ;	
/*Logical addr used to access physc. addr 0		*/
	unsigned long int	*log_0_addr ;		
	/*Flag indicating if system supports bit 31 addressing	*/
	/*	- 0x00 = not supported				*/
	/*	- 0xFF = supported				*/
	unsigned char		bit_31_support ;		
	/*Displacement to area that is not all 0's or 1's	*/
	unsigned long int	non_1s_displc ;			
	/*Saved contents of physical location w/bit 31 set	*/
	unsigned long int	physc_hi_sav_val ;		
	/*Saved contents of physical location 0			*/
	unsigned long int	physc_lo_sv_val ;		
	/*Contents of physical location 0			*/
	unsigned long int	physc_lo_val ;			

	/*General purpose variables*/
	unsigned int		i ;
	unsigned int		j ;
	unsigned long   flags;

	volatile void *			caddr;


	/*Is bus type parameter as expected ?			*/
	if ( (bus_type != 1) && (bus_type != 2) ){							
		/*If no, exit indicating did not find hole		*/
		/*Set returned mem hole addr to null			*/
		*avail_addr = 0 ;				
		/*Return to call with bad parm status			*/
		return ((unsigned char)0x02) ;			
	}

	/*Backgound Table of Memory Hole addresses to probe for availability*/
	/*Initialize Memory Hole address array to all 0's	*/
	/*Set Memory Hole address = null (end of table)		*/
	for ( i = 0; i < HOLE_TABLE_SIZE; ++i )			
		ist.sys_isa_holes[i] = 0 ;			

	/*Backgound Status Table (holds results of memory hole probe)*/
	/*Initialize Memory Hole Status array to all 0's	*/
	/*Set corresponding Memory Hole status = not checked	*/
	for ( i = 0; i < HOLE_TABLE_SIZE; ++i )			
		ist.sys_isa_hole_status[i] = 0 ;		


	/*Set up and probe for availability 16k areas between 640k & 1M								*/
	/*Loop to move probe addresses that are btw 640k & 1M	*/
	for ( i = 0; i < oper_input.isa_num_ents; ++i )	{
		/*Is index about to overflow table ?			*/
		/*If yes, leave last entry = null (0)			*/
		if ( i >= (HOLE_TABLE_SIZE-1) )			
			break ;					
		/*Move an addr to probe from "space.c" 640k-1M array*/
		ist.sys_isa_holes[i] = oper_input.isa_addr_tbl[i] ; 
	}
	/*Loop to probe areas & set status till exhaust array	*/
	/*If Null entry in table, search exhausted		*/
	for ( pa_index = 0 ;ist.sys_isa_holes[pa_index] != 0 ;++pa_index ){
		/*Should issue progress messages ?*/
		if ( !oper_input.quiet_mode ){						
       			printk("CHECK 16k ADDRESS SPACE %x FOR AVAILABILITY \n", 
				 (unsigned int) ist.sys_isa_holes[pa_index] ) ;
		}
		ist.sys_isa_hole_status[pa_index] = occupied_check (ist.sys_isa_holes[pa_index]) ;
	}

/*Completed searching all addresses between 640k & 1M, set up and 
determine if system supports bit 31 addressing
NOTE: Some ISA systems use a dual memory bus architecture such 
that when physical addr bit 31 is set, the low 24 bits
are sent out the ISA bus instead of system's local memory bus.*/

	/*Should probe mem addrs that have bit 31 set ?	*/
	/*If yes, ISA bus w/bit 31 chk enabled, chk if sys	*/
	/* supports bit 31 addressing scheme		*/
	if ( (bus_type == 1) && (oper_input.isa_bit_31_probe == 1) ) {
#if (EQNX_VERSION_CODE > 66304)

		if ((caddr = ioremap(B31_PHYSC_ADDR, 0x1000)) == (void *) NULL){
                	printk("EQUINOX: find_memory_hole: B31, Failed to map physical address space = %x\n",
                        (unsigned int) B31_PHYSC_ADDR); 
                	return((unsigned char)0x80);
        	}
#else
		if((caddr = vmalloc(0x1000)) == (char *) NULL){
			printk("EQUINOX: Failed to allocate virtual address space during ISA config\n");
			return((unsigned char)0x80);
		}
		if ((rc = remap_page_range((TASK_SIZE + ((unsigned long) caddr)),
			B31_PHYSC_ADDR, 0x1000, PAGE_IOMEM))) {
			printk("EQUINOX: Failed to map physical address space = %x, errno = %d\n",
				(int) B31_PHYSC_ADDR, rc); 
			return((unsigned char)0x80);
		}
#endif
		/*Did successfully map probe address ?		*/
		if ( !caddr ){
			/*If no, exit indicating did not find hole	*/
			/*Set returned mem hole addr to null		*/
			*avail_addr = 0 ;				
			/*Return to caller with error status		*/
			return ((unsigned char)0x80) ;			
		}
		else
		  log_bit_31_addr = (unsigned long int *) caddr;
#if (EQNX_VERSION_CODE > 66304)
		caddr = PHYSC_ADDR_0;
#else
		if((caddr = vmalloc(0x1000)) == (char *) NULL){
			printk("EQUINOX: Failed to allocate virtual address space during ISA config\n");
			return((unsigned char)0x80);
		}
		if ((rc = remap_page_range((TASK_SIZE + ((unsigned long) caddr)),
			PHYSC_ADDR_0, 0x1000, PAGE_IOMEM))) {
			printk("EQUINOX: Failed to map physical address space = %x, errno = %d\n",
				(int) PHYSC_ADDR_0, rc); 
			return((unsigned char)0x80);
		}
#endif
		if ( !caddr ){
			/*If no, exit indicating did not find hole	*/
			/*Set returned mem hole addr to null		*/
			*avail_addr = 0 ;				
			/*Return to caller with error status		*/
			return ((unsigned char)0x80) ;			
		}
		else
		  log_0_addr = (unsigned long int *) caddr;
		/*Init flag indicating bit 31 addr not supported*/
		bit_31_support = 0 ;					
		/*Detrm. displc to area in low mem that	*/
		/* is not all 0's or 1's		*/
		for ( non_1s_displc = 0; non_1s_displc < 8; ++non_1s_displc ){
			/*Does loc = all 0's or all 1's ?	*/
			/*If no, use displacement to test bit 31*/
			if ( (*(log_0_addr+non_1s_displc) != 0) &&		
			     (*(log_0_addr+non_1s_displc) != 0xFFFFFFFF ) )
				break ;						
		}
		/*Did find non-0's/1's location in low mem ?	*/
		if ( non_1s_displc < 8 ){
			/*If yes, test if bit 31 addressing functional ?*/
			/*Get 32 bit value @ physc addr 0*/
			physc_lo_sv_val  = *(log_0_addr+non_1s_displc) ;	
			/*Get val @ physc addr 0x80000000*/
			physc_hi_sav_val = *(log_bit_31_addr+non_1s_displc) ;	
			/*Did hi addr actually addr 0 ?	*/
			if ( physc_lo_sv_val != physc_hi_sav_val ){
				save_flags(flags);
				cli();
				/* clobber low memory		*/
				/*Write test pattern @ hi addr	*/
				*(log_bit_31_addr+non_1s_displc) = 0xEE11DD22 ;		
				/*Read physc 0*/
				physc_lo_val = *(log_0_addr+non_1s_displc) ;		

				/*Did test pat appear @0 or	*/
				/* did low mem get clobbered ?	*/
				if ( (physc_lo_val == 0xEE11DD22)    || 		
				     (physc_lo_val != physc_lo_sv_val) ){
					/*If yes, sys not support bit 31*/
					/*Restore value @0		*/
					*(log_0_addr+non_1s_displc) = physc_lo_sv_val ;	
					/*Enable interrupts		*/
       					restore_flags(flags); 
				}
				else {
					/*Restore bit value @ 0x80000000*/
				  *(log_bit_31_addr+non_1s_displc) = physc_hi_sav_val ;	
       				  restore_flags(flags); /*Enable interrupts*/
				  /*Indicate bit 31 addr supported*/
				  bit_31_support = 0xFF ;
				}
			}
#if (EQNX_VERSION_CODE < 66304)
			vfree((void *) log_0_addr);
#endif
			vfree(log_bit_31_addr);

			/*If System supports bit 31 ISA bus addressing 
			(dual memory addressing architecture), set up and probe for*/
			/*availability 16k areas with bit 31 set*/
			/*Does system support dual addr arch. ?	*/
			if ( bit_31_support == 0xFF ){
				/*If yes, bld tbls/arrays w/bit 31 addrs*/
				/*Add addrs to tbl*/
				for ( i = 0, j = pa_index; 
					i < oper_input.isa_num_bit_31_ents; ++i, ++j) {
					/*Is index about to overflow table ?	*/
					/*If yes, lv last entry = 0	*/
					if ( j >= (HOLE_TABLE_SIZE-1) )		
						break ;					
					ist.sys_isa_holes[j] = oper_input.isa_bit_31_tbl[i];
				}
				/*Loop to probe memory areas	*/
				for (; ist.sys_isa_holes[pa_index] != 0; ++pa_index ){
					/*Should issue progress msgs ?	*/
					if ( !oper_input.quiet_mode ) /*If yes*/
       						printk("31bit: CHECKING 16k ADDRESS SPACE %x FOR AVAILABILITY \n", 
						 (unsigned int) ist.sys_isa_holes[pa_index] ) ;
						ist.sys_isa_hole_status[pa_index] = 
							occupied_check (ist.sys_isa_holes[pa_index]) ;
				}
			}/*End of bit 31 adrress area probing	*/
		}/*End of bit 31 functionality testing		*/
	}/*End of ISA bit 31 check				*/
	if ( !oper_input.quiet_mode )/*Should issue progress msgs ?*/
          printk("\n" );/*If yes, position cursor at new line*/

	/*Memory table/array completed, determine if have identified 
	at least 1 available memory area/hole, and if so, return	*/
	/*address of 1st found entry in table.*/

	/*Search Status Table till:				*/
	/*	o Find an available area  OR			*/
	/*	o Exhaust table					*/
	for ( pa_index = 0 ;	
	      ist.sys_isa_hole_status[pa_index] != 0xF0 &&	
	      ist.sys_isa_hole_status[pa_index] != 0;		
	      ++pa_index ) ;
	/*Did find an unoccupied 16k area ?			*/
	if ( ist.sys_isa_hole_status[pa_index] == 0xF0 ){							
		/*If yes, return physical address to caller		*/
		/*Return physical address of unoccupied 16k area*/
		*avail_addr = ist.sys_isa_holes[pa_index];		
		/*Return to caller with "found" status		*/
		return ((unsigned char)0x00) ;				
	}

	/*Searched of entire Probe array failed to identify an available 
	16k memory region, return "no avail memory hole" status*/
	/*Set returned mem hole addr to null			*/
	*avail_addr = 0 ;					
	/*Return to caller with no avail mem hole error status	*/
	return ((unsigned char)0xFF) ;				
}  /* find_memory hole *//*END of FIND 16k MEMORY HOLE FUNCTION*/

/****************************************************************************************
 *                                                                              	*
 *			CHECK IF 16K ADDRESS SPACE OCCUPIED				*
 *                                                                              	*
 * PURPOSE: To determine if a 16k area of memory (ISA address space) is empty i.e. not	*
 *	    occupied by ROM/RAM or any other ISA device.				*
 *											*
 * NOTES: o Function unconditionally DISABLES/ENABLES INTERRUPTS !!!!!!			*
 *	  o Function MAY WRITE to memory area being checked				*
 *											*
 * CALL:  unsigned char occupied_check (unsigned long int	physc_base_addr) 	*
 *											*
 *			where: physc_base_addr = Physical base addr of 16k memory area	*
 *						 to check if occupied			*
 *											*
 *											*
 * RETURN:  - Status 16k area (unsigned char):						*
 *		o 0x01 = Address space occupied by ROM					*
 *		o 0x02 = Address space occupied by RAM					*
 *		o 0x80 = Could not map/de-map physical (probe) address			*
 *		o 0xF0 = Address space available					*
 *											*
 ****************************************************************************************/

static unsigned char occupied_check (unsigned long int physc_base_addr )

{

	/*Logical base address of memory area to probe		*/
	unsigned char *log_addr ;			
	/*General purpose variable used for return status	*/
	/*Function's return status parameter			*/
	unsigned char		return_status ;			
	/*Ctr used to srch for RAM within 16k memory space	*/
	unsigned int		srch_ctr ;			
	/*Save contents of memory 				*/
	unsigned long int	sv_mem_0 ;			
	unsigned long   flags = 0;
	/*Save contents of memory 				*/
	unsigned long int	sv_mem_1 ;			
	void * 			caddr;
	volatile unsigned int	*mem_rdp;
	volatile unsigned int	mem_rd;
#if (EQNX_VERSION_CODE < 66304)
	int rc;
#endif
	unsigned int		rd_ctr;
#ifdef DEBUG
	char prt_str[200];
#endif

#if (EQNX_VERSION_CODE > 66304)
#if (EQNX_VERSION_CODE < 131328)
	if (physc_base_addr > 0x100000){
#endif

		if ((caddr = ioremap(physc_base_addr, 0x4000)) == (void *) NULL){
               		printk("EQUINOX: occupied check: Failed to map physical address space = %x\n",
                       	(unsigned int) physc_base_addr); 
               		return((unsigned char)0x80);
        	}
#if (EQNX_VERSION_CODE < 131328)
	} else
		caddr = (void *) physc_base_addr;
#endif		
#else
	if((caddr = vmalloc(0x4000)) == (char *) NULL){
		printk("EQUINOX: Failed to allocate virtual address space during ISA config\n");
		return((unsigned char)0x80);
	}
	if ((rc = remap_page_range((TASK_SIZE + ((unsigned long) caddr)),
		physc_base_addr, 0x4000, PAGE_IOMEM))) {
		printk("EQUINOX: Failed to map physical address space = %x, errno = %d\n",
			(unsigned int) physc_base_addr, rc); 
		return((unsigned char)0x80);
	}
#endif

	if ( !caddr )/*Did successfully map probe address ?*/
	/*Return to caller with can't map physc addr error	*/        
		return ((unsigned char)0x80);
	else
		log_addr = (unsigned char *) caddr;
	/*Determine if mem area appears unoccupied	*/
	return_status = passive_occupied_chk (physc_base_addr, log_addr); 
#ifdef DEBUG
sprintf(prt_str,"after passive_occuped_check with params, phy addr = %x log addr = %x\n", 
	(unsigned int) physc_base_addr, (unsigned int) log_addr);
printk(prt_str);
#endif
	/*Is memory area occupied ?				*/
	if ( return_status == 0xF0 ){							
	/*If no, mem area does not appear occupied, attempt to	*/
	/*write & read area to assure not occupied		*/
		/*Loop to perform wrt/rd operations in 1k inc's	*/
		for ( srch_ctr = 0; srch_ctr < 16; ++srch_ctr )	{
			/*Read & Save location 0 and 1	*/
			sv_mem_0 = *((unsigned long int *)(log_addr+(1024*srch_ctr))) ;	 
			sv_mem_1 = *((unsigned long int *)(log_addr+(1024*srch_ctr)+4)); 
        		/*Disable interrupts in case	*/
			save_flags(flags);
			cli();
			/*write does clobber mem	*/
			/*Write location 0		*/
			*((unsigned long int *)(log_addr+(1024*srch_ctr))) =    	 
			  (unsigned long int)0xAAAA5555 ;
			/*Write location 4, set data	*/
			/*lines high			*/
			*((unsigned long int *)(log_addr+(1024*srch_ctr)+4)) =		 
			  (unsigned long int)0xFFFFFFFF ;		 	 	 
			
			for ( rd_ctr = 32; rd_ctr != 0; --rd_ctr )
			{
				mem_rdp = (volatile unsigned int *)(log_addr+(1024*srch_ctr));
				mem_rd = *mem_rdp;
				if ( mem_rd == (unsigned long int)0xFFFFFFFF )
			  	  break;
			}
			if ( rd_ctr == 0 ) {
				/*Yes, area occupied by RAM	*/
				/*Restore		*/
				*((unsigned long int *)(log_addr+(1024*srch_ctr))) = sv_mem_0 ;	 
				*((unsigned long int *)(log_addr+(1024*srch_ctr)+4)) = sv_mem_1; 
       				restore_flags(flags); /*Enable interrupts*/
				return_status = ((unsigned char)0x02) ;	/*Return area occupied*/
				break ;
			}/*by RAM status	*/
       			restore_flags(flags); /*Enable interrupts	*/
		}
	}/*End of write/read check loop				*/

    /*Determination has been made whether caller's memory area 
	is occupied, free memory & return status to caller	*/
#if ((EQNX_VERSION_CODE > 66304) && (EQNX_VERSION_CODE < 131328))
	if (physc_base_addr > 0x100000)
#endif
		iounmap(caddr);
	return (return_status) ;/*Return status*/
}  /* occupied_check */


/****************************************************************************************
 *                                                                              	*
 *		     CHECK IF 16K ADDRESS SPACE OCCUPIED (NON_WRITE)			*
 *                                                                              	*
 * PURPOSE: To determine if a 16k area of memory (ISA address space) is empty i.e. not	*
 *	    occupied by ROM/RAM or any other ISA device by only reading memory.		*
 *											*
 * NOTES: o Function unconditionally DISABLES/ENABLES INTERRUPTS !!!!!!			*
 *	  o Function performs check by only searching ROM array & reading slected memory*
 *	    locations (in caller's address space).  Function DOES NOT WRITE to memory	*
 *	    area. 									*
 *											*
 * CALL:  unsigned char passive_occupied_chk (unsigned long int   physc_addr,		*
 *					      unsigned char	  *log_addr) 		*
 *											*
 *			where: physc_addr = Physical addr of 16k memory area to check	*
 *					    if occupied					*
 *			       log_addr   = Logical addr of 16k memory area to check if	*
 *					    occupied					*
 *											*
 *											*
 * RETURN:  - Status 16k area (unsigned char):						*
 *		o 0x01 = Address space occupied by ROM					*
 *		o 0x02 = Address space occupied by RAM					*
 *		o 0x80 = Could not map/de-map ROM area					*
 *		o 0xF0 = Address space not occupied by ROM & does not appear to be 	*
 *			 occupied by RAM						*
 *											*
 ****************************************************************************************/

static
unsigned char	passive_occupied_chk (unsigned long int	   physc_addr,
		      unsigned char *log_addr   )

{
	/*General purpose variable used for return status	*/
	unsigned char		uchar_ret_val ;			
	/*Prob addr's index into ROM array			*/
	unsigned int		rom_array_index ;		
	/*Ctr used to srch for ROM/RAM within 16k probe area	*/
	volatile unsigned int		srch_ctr ;			
	volatile unsigned int		*mem_rdp;
	volatile unsigned int		mem_rd;
	volatile unsigned int		rd_ctr;


	/*Did initialize ROM Array successfully ?	*/
	if ( (uchar_ret_val = rom_check ( )) != 0 )		
		/*If no, exit with error status			*/
		return ( uchar_ret_val ) ;			
	/* is addr to check in VGA graphic area? */
	if ( ((physc_addr & 0xfffff) >= 0xa0000) &&		
	     ((physc_addr & 0xfffff) < 0xb0000) ){
		rom_array_index = (0xc0000 - ROM_AREA_BASE)/2048;
		if ( rom_array[rom_array_index] != 0 )
			/* if yes, return arrea occupied by RAM */
			return( (unsigned char) 0x02 );		
	}
	/*Is addr to check in system ROM area ?			*/
	if ( ((physc_addr & 0xFFFFF) >= ROM_AREA_BASE) &&	
	     ((physc_addr & 0xFFFFF) <  ROM_AREA_LMT) ) {
		/*If yes, check if ROM occupies any portion of 16k area	*/
		/*Mem area's index into ROM array	*/
		rom_array_index = ((physc_addr & 0xFFFFF) - ROM_AREA_BASE)/2048 ; 
		/*Develop # ROM loc's in 16k spc		*/
		srch_ctr = MAX_ROM_LOCS - rom_array_index ;		
		if ( srch_ctr > 8 ) /*Is addr at "top" of ROM area ?		*/
			/*If no, then max # ROM locations to check	*/
			srch_ctr = 8 ;
		/*Loop thru ROM array chk'g if ROM occupies any	*/
		/* portion of caller's 16k memory area		*/
		for ( ; srch_ctr != 0; --srch_ctr ){							
			/*Is ROM present within area ?			*/
			if ( rom_array[rom_array_index++] != 0 )	
			/*If yes, return area occupied by ROM status 	*/
				return ((unsigned char)0x01) ;		
		}
	}/*End of ROM check loop					*/

/*Caller's 16k block is not occupied by ROM, 
set up and check if occupied by RAM*/
#ifdef DEBUG
printk("checking ram for address %x, logical addr = %x\n", 
	(unsigned int) physc_addr, (unsigned int) log_addr);
#endif
	/*Loop to perform read operations on area		*/
	for ( srch_ctr = 0; srch_ctr < 16; ++srch_ctr )	{
		for ( rd_ctr = 32; rd_ctr != 0; --rd_ctr ) {
			mem_rdp = (volatile unsigned int *)(log_addr+(1024*srch_ctr));
			mem_rd = *mem_rdp;
			if ( mem_rd == (unsigned long int) 0xffffffff )
			  break;
		}
		if ( rd_ctr == 0 )
		/*If no, Return area occupied by RAM	*/
			return ((unsigned char)0x02) ;	
	}

/*Caller's 16k block does not appear to be 
occupied by RAM, return status to caller*/
	return ((unsigned char)0xF0) ;

}  /* passive_occupied_chk */

/****************************************************************************************
 *                                                                              	*
 *				      ROM SEARCH					*
 *                                                                              	*
 * PURPOSE: To build byte mapped array of memory areas (address space) occupied by ROM.	*
 *											*
 * NOTES: 										*
 *											*
 * CALL:  unsigned char rom_check (void)						*
 *											*
 *											*
 * RETURN:  - Function status:								*
 *		o 0x00 = ROM array successfully built					*
 *		o 0x80 = Could not map/de-map physical (probe) address			*
 *											*
 ****************************************************************************************/

static unsigned char	rom_check (void) 
{
	/*Logical base address of ROM memory region		*/
	unsigned char  	*rom_probe_base ;		
	/*ROM array index					*/
	unsigned int		ra_index ;			
	/*Logical base address of memory area to probe		*/
	unsigned char  	*log_probe_addr ;		
	/*Size of a found ROM (in units of 2k)			*/
	unsigned short int	rom_size ;			
	void *caddr;
#if (EQNX_VERSION_CODE < 66304)
	int rc;
#endif


	/*Has ROM Array been previously built ?			*/
	if ( rom_array_built == 0xFF )				
	/*If yes, return to caller w/good status		*/
		return ((unsigned char)0) ;			

#if (EQNX_VERSION_CODE > 66304)
#if (EQNX_VERSION_CODE > 131327)
		if ((caddr = ioremap(ROM_AREA_BASE, (MAX_ROM_LOCS * ROM_BNDRY_SZ))) == (void *) NULL){
               		printk("EQUINOX: rom check: Failed to map physical address space = %x\n",
                       	(unsigned int) ROM_AREA_BASE); 
               		return((unsigned char)0x80);
        	} /* Pat */
#else
	caddr = (void *) ROM_AREA_BASE;
#endif	
#else
	if((caddr = vmalloc(MAX_ROM_LOCS * ROM_BNDRY_SZ)) == (char *) NULL){
		printk("EQUINOX: Failed to allocate virtual address space during ISA config\n");
		return((unsigned char)0x80);
	}
	if ((rc = remap_page_range((TASK_SIZE + ((unsigned long) caddr)),
		ROM_AREA_BASE, MAX_ROM_LOCS * ROM_BNDRY_SZ, PAGE_IOMEM))) {
		printk("EQUINOX: Failed to map physical address space = %x, errno = %d\n",
			(unsigned int) ROM_AREA_BASE, rc); 
		return((unsigned char)0x80);
	}
#endif
	/*Did successfully map ROM area ?			*/
	if ( !caddr )						
	/*If no, exit indicating mapping failure	*/
		return ((unsigned char)0x80) ;				
	else
		rom_probe_base = (unsigned char *) caddr;
	/*Set up & initialize ROM array*/
	/*Set Up & srch all addresses where ROM might reside	*/
	for ( ra_index = 0; ra_index < MAX_ROM_LOCS; )		
	{
		/*Pt to nxt possible ROM probe addr		*/
		log_probe_addr = rom_probe_base+(ROM_BNDRY_SZ * ra_index); 
		/*Is ROM signature at probe address ?	*/
		if ( (*log_probe_addr == (unsigned char)0x55) && 	   
		     (*(log_probe_addr+1) == (unsigned char)0xAA) ) {

			/*If yes, found ROM signature			*/
			/*Get ROM's sz in 2k units & add another*/
			/* unit if not mod 2k			*/
			rom_size = *(log_probe_addr+2) >> 2 ;			
			if ( (*(log_probe_addr+2) & 0x03) != 0 ) ++rom_size; 	
			/*Is number of blocks sane ?			*/
			if ( rom_size != 0 ){						
			/*If yes, then found ROM, update ROM array	*/
				for ( ; (rom_size != 0) && (ra_index < MAX_ROM_LOCS); --rom_size)
				/*Indicate ROM present in 2k area	*/
					rom_array[ra_index++] = 0xFF ;		
			}
			else 
			/*Found ROM signature but # blocks bad, continue*/
			/* as if no ROM found				*/
				rom_array[ra_index++] = 0 ;		
		}/*End of logic if ROM				*/
		else /*ROM signature not found at Probe addr		*/
		/*Indicate no ROM present & pt to nxt array elm.*/
			rom_array[ra_index++] = 0 ;			
	}/*End of loop indentifying memory areas occupied by ROM	*/
#if ((EQNX_VERSION_CODE < 66304) || (EQNX_VERSION_CODE > 131327))
	iounmap(caddr);
#endif /* Pat */

	rom_array_built = 0xFF ;/*Set falg indicating array built*/
	return ((unsigned char)0) ;
}  /* ROM_CHECK */

#endif	/* ISA_ENAB */
